/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004, Diego Pettenò                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "knetload.h"
#include "knetproc.h"
#include "settings.h"
#include "interfaces.h"
#include "stattray.h"
#include "statpopup.h"

#include <kcombobox.h>
#include <kconfigdialog.h>
#include <kaction.h>
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>

KNetLoad::KNetLoad()
	: LoadApp(false), m_proc(new KNetProc()) 
{
	m_popup = new StatPopup(0, "stat_popup");
	setMainWidget(m_popup);

	InterfacesConfig *ifcfg = new InterfacesConfig(m_popup, "Interfaces");
	const QStringList &ifaces = KNetProc::loadInterfaces(true);
	QStringList::const_iterator enddev = ifaces.end();
	for(QStringList::const_iterator it = ifaces.begin(); it != enddev; it++)
		ifcfg->kcfg_IfName->insertItem(KNetProc::getInterfaceIcon(*it), *it);
	
	m_cfgdialog->addPage(ifcfg, i18n("Interfaces"), "network");
	
	// Create system tray windows.
	m_graphIn = new StatTray(m_popup, "graph_in");
	m_graphOut = new StatTray(m_popup, "graph_out");
	
	// Set up actions and read the config file.
	setupActions();
	connect(m_actClearHistory, SIGNAL(activated()), m_graphIn->graph(), SLOT(clear()));
	connect(m_actClearHistory, SIGNAL(activated()), m_graphOut->graph(), SLOT(clear()));
	
	initPopupMenu(m_graphIn->contextMenu());
	initPopupMenu(m_graphOut->contextMenu());

	reloadSettings();
	startUpdates();
}

KNetLoad::~KNetLoad()
{
	delete m_popup;
}

void KNetLoad::reloadSettings()
{
	QString newDevice = Settings::ifName().stripWhiteSpace();
	if (newDevice.isEmpty())
	{
		KMessageBox::error(0, i18n("The device name cannot be empty."));
		return;
	} else {
		m_proc->setDevice(newDevice);
	}
	
	m_graphIn->graph()->setUpperColor(Settings::color1());
	m_graphIn->graph()->setStyle((StatGraph::Style)Settings::style());
	m_graphIn->graph()->setGridPace( Settings::grid() ? 6 : 0 );
	m_graphIn->graph()->setBgColor( Settings::transparentBackground() ? StatGraph::invalidColor : Settings::colorBackground() );
	m_graphIn->graph()->setLabel( Settings::labelled() ? "IN" : QString::null );
	m_graphIn->graph()->redraw();
	m_graphIn->show();
	
	m_graphOut->graph()->setUpperColor(Settings::color2());
	m_graphOut->graph()->setStyle((StatGraph::Style)Settings::style());
	m_graphOut->graph()->setGridPace( Settings::grid() ? 6 : 0 );
	m_graphOut->graph()->setBgColor( Settings::transparentBackground() ? StatGraph::invalidColor : Settings::colorBackground() );
	m_graphOut->graph()->setLabel( Settings::labelled() ? "OUT" : QString::null );
	m_graphOut->graph()->redraw();
	m_graphOut->show();
	
	LoadApp::reloadSettings();
}

void KNetLoad::takeReading()
{
	kdDebug() << "KNetLoad::takeReading()" << endl;
	m_proc->readLoad();
	
	// Rates in bits per second.
	float m_bitRateIn = m_proc->recentBytesIn() * 8.0 /
		(((float) Settings::timeout()) / 1000.0);
	float m_bitRateOut = m_proc->recentBytesOut() * 8.0 /
		(((float) Settings::timeout()) / 1000.0);

	// Convert to percentages.
	char upperIn = (char) (100.0 * m_bitRateIn / ((float) Settings::scaleIn()));
	char upperOut = (char) (100.0 * m_bitRateOut /
		((float) (Settings::scaleOut() ? Settings::scaleOut() : Settings::scaleIn() )));

	if (upperIn < 0)
		upperIn = 0;
	if (upperOut < 0)
		upperOut = 0;
	if (upperIn > 100)
		upperIn = 100;
	if (upperOut > 100)
		upperOut = 100;
	
	m_graphIn->graph()->addPercentReading(upperIn, 0, Settings::soft());
	m_graphOut->graph()->addPercentReading(upperOut, 0, Settings::soft());

	if ( !m_popup->isVisible() ) return;
	
	if ( Settings::speedUnit() == 0 ) // bits per second
	{
		float currBitIn = (int)((m_bitRateIn + 50) / 100) / 10.0;
		float currBitOut = (int)((m_bitRateOut + 50) / 100) / 10.0;
		m_popup->setText(i18n(
			"Current In: %1 Kb/s, Total In: %2 MB.\n"
			"Current Out: %3 Kb/s, Total Out: %4 MB.")
			.arg(currBitIn).arg(m_proc->totalMbIn())
			.arg(currBitOut).arg(m_proc->totalMbOut()));
	} else { // bytes per second
		float currByteIn = (int)((m_bitRateIn/8 + 50) / 100) / 10.0;
		float currByteOut = (int)((m_bitRateOut/8 + 50) / 100) / 10.0;
		m_popup->setText(i18n(
			"Current In: %1 KB/s, Total In: %2 MB.\n"
			"Current Out: %3 KB/s, Total Out: %4 MB.")
			.arg(currByteIn).arg(m_proc->totalMbIn())
			.arg(currByteOut).arg(m_proc->totalMbOut()));
	}
	m_popup->resizeToText();
}

#include "knetload.moc"
